package ms.warpzone.warpzonestatuswidget;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.TargetApi;
import android.os.Build;
import android.os.StrictMode;
import android.text.format.DateFormat;
import android.util.Log;

public class StatusLoader {

	private static String loadJSONPage(String url) {
		try {
			URL uri = new URL(url);
			HttpURLConnection connection = (HttpURLConnection) uri.openConnection();
			BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
			
			String result = "";
			String line = reader.readLine();
			if(!line.contains("{"))
			{
				Log.d("loadJSONPage", "no JSON found on page.");
				return null;
			}
			do {
				result += line + "\n";
			} while ((line = reader.readLine()) != null);
			
			return result;
		} catch (Exception e) {
			if(e != null)
				Log.d("loadJSONPage", e.getMessage());
			return null;
		}
	}
	
    @TargetApi(Build.VERSION_CODES.GINGERBREAD)
    private static void allowNetworkingInMainThread() {
    	//Allow Networking in this thread.
    	StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
	  	StrictMode.setThreadPolicy(policy);
    }
	
	public static int getStatusImageID() {
		
		try {
			allowNetworkingInMainThread();
		}
		catch(Exception e)
		{
			Log.d("getStatusImageID", "allowNetworkingInMainThread not supported.");
		}
		
        String content = loadJSONPage("http://warpzone.ms/api/status.php");
        if(content == null) {
        	Log.d("getStatusImageID", "Could not load status page.");
        	return R.drawable.warpzone_logo_small;
        }
        
    	JSONObject json;
		try {
			json = new JSONObject(content);
			
			long record_time = json.getLong("timestamp");
			long now = System.currentTimeMillis()/1000L;
			
        	if((now - record_time) > 600)
        	{
        		Log.d("getStatusImageID", "Status record is too old.");
        		return R.drawable.warpzone_logo_orange_small;
        	}
        	
        	if(json.getInt("tuerOffen") != 0) 
        	{
        		return R.drawable.warpzone_logo_green_small;
        	}
        	
        	return R.drawable.warpzone_logo_red_small;
		} catch (JSONException e) {
			Log.d("getStatusImageID", "Could not parse status JSON.");
			return R.drawable.warpzone_logo_orange_small;
		}
	}
}